/*
---
description: Supplies a Mock Event object for use on fireEvent

license: MIT-style

authors:
- Arieh Glazer

requires:
- core/1.2.4 : [Native/Event]

provides: [Event.Mock]

...
*/
/*!
Copyright (c) 2010 Arieh Glazer

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE
*/
define([], function () {
    (function ($, window, undef) {

        /**
         * creates a Mock event to be used with fire event
         * @param Element target an element to set as the target of the event - not required
         *  @param string type the type of the event to be fired. Will not be used by IE - not required.
         *
         */
        Event.Mock = function (target, type) {
            var e = window.event;

            type = type || 'click';

            if (document.createEvent) {
                e = document.createEvent('HTMLEvents');
                e.initEvent(
                    type, //event type
                    false, //bubbles - set to false because the event should like normal fireEvent
                    true //cancelable
                );
            } else if (document.createEventObject) {
                // Internet explorer 8?
                e = document.createEventObject();
            }
            // No longer works in 1.4.5
            //e = new Event(e);
            e = new DOMEvent(e, self.getWindow());
            e.target = target;
            return e;
        };

    })(document.id, window);
    return Event.Mock;
});